/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: blog
 * Version: 1.09.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.blog.dao.impl;

import java.sql.Types;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;

import com.kosmos.blog.bean.BlogBean;
import com.univ.objetspartages.dao.AbstractFicheDAO;

public class BlogDAO extends AbstractFicheDAO<BlogBean> {

    private static final Logger LOG = LoggerFactory.getLogger(BlogDAO.class);

    public BlogDAO() {
        tableName = "BLOG";
    }

    /**
     * Sélection un blog par sa rubrique de rattachement, sa langue ainsi que son état.
     * @param codeRubrique la rubrique du blog
     * @param langue la langue
     * @param etat l'état
     * @return le blog correspondant, ou null si aucun résultat
     */
    public BlogBean selectByCodeRubriqueLangueEtat(String codeRubrique, String langue, String etat){
        initColumns();
        MapSqlParameterSource parameterSource = new MapSqlParameterSource();
        String request = " WHERE T1.ETAT_OBJET = :etat AND T1.CODE_RUBRIQUE = :codeRubrique AND T1.LANGUE = :langue";
        parameterSource.addValue("etat", etat, Types.VARCHAR);
        parameterSource.addValue("codeRubrique", codeRubrique, Types.VARCHAR);
        parameterSource.addValue("langue", langue, Types.VARCHAR);
        String query = String.format(SELECT_DISTINCT, columnsNameForRequest, tableName) + request;
        List<BlogBean> listBlogBean = select(query, parameterSource);
        if(listBlogBean == null || listBlogBean.isEmpty()){
            return null;
        }
        if(listBlogBean.size() > 1){
            LOG.warn(String.format("Plusieurs blogs pour la rubrique ayant pour code, %s", codeRubrique));
        }
        return listBlogBean.get(0);
    }

}
