/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: blog
 * Version: 1.09.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.blog.misc;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.database.OMContext;
import com.univ.objetspartages.om.AbstractRechercheExterne;
import com.univ.objetspartages.util.CritereRecherche;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.utils.RechercheFicheHelper;

import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereDate;
import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereDateAvecLibelleFormate;
import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereTexteNonVideFormater;
import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereThematique;

public class RechercheBlog extends AbstractRechercheExterne {

    /**
     * Prepare le formulaire de recherche.
     */
    @Override
    public boolean preparerRECHERCHE(final InfoBean infoBean, final OMContext ctx) throws Exception {
        boolean specific = false;
        final String objet = StringUtils.upperCase(infoBean.getString("OBJET"));
        if ("ARTICLEBLOG".equals(objet)) {
            specific = true;
        } else if ("BLOG".equals(objet)) {
            infoBean.set("LISTE_THEMATIQUES", LabelUtils.getLabelCombo("04", ctx.getLocale()));
            specific = true;
        }
        return specific;
    }

    /**
     * Traite la recherche.
     */
    @Override
    public boolean traiterRECHERCHE(final InfoBean infoBean, final OMContext ctx) throws Exception {
        boolean rechercheEffectuee = false;
        final String objet = StringUtils.upperCase(infoBean.getString("OBJET"));
        // articleblog
        if ("ARTICLEBLOG".equals(objet)) {
            final List<CritereRecherche> criteres = new ArrayList<>();
            criteres.add(new CritereRecherche("OBJET", objet, StringUtils.EMPTY));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "TITRE"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "KEYWORDS"));
            CollectionUtils.addIgnoreNull(criteres, getCritereDateAvecLibelleFormate(infoBean, "DATE_MISE_EN_LIGNE", "MMMM yyyy"));
            CollectionUtils.addIgnoreNull(criteres, getCritereDate(infoBean, "DATE_DEBUT"));
            CollectionUtils.addIgnoreNull(criteres, getCritereDate(infoBean, "DATE_FIN"));
            criteres.addAll(getCritereDefaut(infoBean));
            infoBean.set(RechercheFicheHelper.ATTRIBUT_INFOBEAN_CRITERES, criteres);
            gestionAncienFront(infoBean, criteres);
            rechercheEffectuee = true;
        }
        // blog
        else if ("BLOG".equals(objet)) {
            final List<CritereRecherche> criteres = new ArrayList<>();
            criteres.add(new CritereRecherche("OBJET", objet, StringUtils.EMPTY));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "TITRE"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "DESCRIPTION"));
            CollectionUtils.addIgnoreNull(criteres, getCritereThematique(infoBean));
            criteres.addAll(getCritereDefaut(infoBean));
            infoBean.set(RechercheFicheHelper.ATTRIBUT_INFOBEAN_CRITERES, criteres);
            gestionAncienFront(infoBean, criteres);
            rechercheEffectuee = true;
        }
        return rechercheEffectuee;
    }

    @Override
    public List<String> getCriteresRequete(final String objet, final boolean listeIncluse) {
        final List<String> criteres = new ArrayList<>();
        if (objet.equals("BLOG")) {
            criteres.add("TITRE");
            criteres.add("DESCRIPTION");
            criteres.add("THEMATIQUE");

        }
        if (objet.equals("ARTICLEBLOG")) {
            criteres.add("KEYWORDS");
            criteres.add("DATE_MISE_EN_LIGNE");
            criteres.add("DATE_DEBUT");
            criteres.add("DATE_FIN");
        }
        return criteres;
    }
}
