/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: blog
 * Version: 1.09.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.blog.om;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.StringTokenizer;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.blog.bean.BlogBean;
import com.kosmos.datasource.sql.utils.SqlDateConverter;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.om.AbstractFicheRestriction;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.InfosRubriques;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.ContexteDao;
import com.univ.utils.RequeteUtil;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseOrderBy.SensDeTri;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.condition.ConditionList;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;

/**
 * Classe representant un objet blog.
 */
@FicheAnnotation(isEncadreRechercheEmbarquable = false, isEncadreRecherche = false)
public class Blog extends AbstractFicheRestriction<BlogBean> {

    /**
     *
     */
    private static final long serialVersionUID = -4013018031894028438L;

    /**
     * Renvoie le libelle a afficher (methode statique utilisee pour les jointures entre fiches).
     */
    public static String getLibelleAffichable(final OMContext ctx, final String codes, String langue) throws Exception {
        String res = StringUtils.EMPTY;
        if (codes == null || codes.length() == 0) {
            return res;
        }
        final Blog blog = new Blog();
        blog.init();
        if (langue == null || langue.length() == 0) {
            langue = "0";
        }
        final StringTokenizer st = new StringTokenizer(codes, ";");
        while (st.hasMoreTokens()) {
            final String code = st.nextToken();
            // On cherche d'abord la version en ligne puis les autres versions
            int count = blog.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) {
                count = blog.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count > 0) {
                if (res.length() > 1) {
                    res += ";";
                }
                if (blog.nextItem()) {
                    res += blog.getLibelleAffichable();
                }
            } else {
                res += "-";
            }
        }
        return res;
    }

    /**
     * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
     */
    public static Blog getFicheBlog(final OMContext ctx, final String code, String langue) throws Exception {
        Blog fiche = null;
        if (code != null && code.length() > 0) {
            if (langue == null || langue.length() == 0) {
                langue = "0"; // francais par defaut
            }
            fiche = new Blog();
            fiche.init();
            int count = fiche.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) // si pas de version en ligne
            {
                count = fiche.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count == 0) // si pas de version dans la langue demandee
            {
                count = fiche.selectCodeLangueEtat(code, StringUtils.EMPTY, EtatFiche.EN_LIGNE.getEtat());
            }
            if (count > 0) {
                fiche.nextItem();
            }
        }
        return fiche;
    }

    public static Hashtable<String, String> getBlogs(final OMContext ctx) throws Exception {
        final Hashtable<String, String> blogs = new Hashtable<>();
        final Blog iBlog = new Blog();
        iBlog.init();
        if (iBlog.select(" where ETAT_OBJET='0003'") > 0) {
            while (iBlog.nextItem()) {
                blogs.put(iBlog.getCode(), iBlog.getLibelleAffichable());
            }
        }
        return blogs;
    }

    /**
     * Initialise l'objet metier.
     */
    @Override
    public void init() {
        super.init();
        setDescription(StringUtils.EMPTY);
        setThematique(StringUtils.EMPTY);
        setArchivage(StringUtils.EMPTY);
    }

    /**
     * Renvoie le libelle a afficher.
     */
    @Override
    public String getLibelleAffichable() {
        return getServiceFiche().getLibelleAffichable(persistenceBean);
    }

    /**
     * Renvoie le libelle pour l'attribut thematique.
     */
    public String getLibelleThematique() {
        return LabelUtils.getLibelle("04", getThematique(), getLangue());
    }

    /**
     * Renvoie la rubrique de la fiche.
     */
    public InfosRubriques getInfosRubrique() {
        InfosRubriques res = new InfosRubriques("");
        if (StringUtils.isNotBlank(getCodeRubrique())) {
            final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(getCodeRubrique());
            if (rubriqueBean != null) {
                res = new InfosRubriques(rubriqueBean);
            }
        }
        return res;
    }

    /**
     * Renvoie la structure de la fiche.
     *
     * @deprecated Utiliser {@link com.univ.objetspartages.services.ServiceStructure#getByCodeLanguage(String, String)}
     */
    @Deprecated
    public StructureModele getInfosStructure() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getByCodeLanguage(getCodeRattachement(), getLangue());
    }

    /**
     * Renvoie les metadonnees de la fiche.
     */
    public MetatagBean getMeta() throws Exception {
        return MetatagUtils.lireMeta(this);
    }

    /**
     * Traitement d'une requete sur l'objet.
     */
    @Override
    public int traiterRequete(final String requete) throws Exception {
        final String titre = RequeteUtil.renvoyerParametre(requete, "TITRE");
        final String thematique = RequeteUtil.renvoyerParametre(requete, "THEMATIQUE");
        final String description = RequeteUtil.renvoyerParametre(requete, "DESCRIPTION");
        final String codeRubrique = RequeteUtil.renvoyerParametre(requete, "CODE_RUBRIQUE");
        final String codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT");
        final String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
        final String nombre = RequeteUtil.renvoyerParametre(requete, "NOMBRE");
        final String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
        final String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
        final String code = RequeteUtil.renvoyerParametre(requete, "CODE");
        return select(titre, thematique, description, codeRubrique, codeRattachement, langue, etatObjet, codeRedacteur, code, nombre);
    }

    /**
     * Renvoie la liste des references a cette fiche.
     */
    @Override
    public String getReferences() {
        return getServiceFiche().getReferences(persistenceBean);
    }

    /**
     * Selection d'une instance de l'objet Blog a partir de l'ensemble des criteres.
     */
    public int select(final String titre, final String thematique, final String description, final String codeRubrique, final String codeRattachement, final String langue, final String etatObjet, final String codeRedacteur, final String code, final String nombre) throws Exception {
        final ClauseWhere where = new ClauseWhere();
        // Traitement du parametre titre
        // Recherche également sur le champs description
        if (StringUtils.isNotEmpty(titre)) {
            final ConditionList listeConditions = new ConditionList();
            listeConditions.setPremiereCondtion(ConditionHelper.rechercheMots("T1.TITRE", titre));
            listeConditions.or(ConditionHelper.rechercheMots("T1.DESCRIPTION", titre));
            where.setPremiereCondition(listeConditions);
        }
        if (StringUtils.isNotEmpty(description)) {
            where.and(ConditionHelper.rechercheMots("T1.DESCRIPTION", description));
        }
        // Traitement du parametre thematique
        if (StringUtils.isNotEmpty(thematique) && !thematique.equals("0000")) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.THEMATIQUE", thematique));
        }
        if (StringUtils.isNotEmpty(codeRattachement)) {
            where.and(ConditionHelper.getConditionStructure("T1.CODE_RATTACHEMENT", codeRattachement));
        }
        if (StringUtils.isNotEmpty(langue) && !"0000".equals(langue)) {
            where.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
        }
        if (StringUtils.isNotEmpty(etatObjet) && !"0000".equals(etatObjet)) {
            where.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", etatObjet));
        }
        if (StringUtils.isNotEmpty(codeRedacteur)) {
            where.and(ConditionHelper.egalVarchar("T1.CODE_REDACTEUR", codeRedacteur));
        }
        if (StringUtils.isNotEmpty(code)) {
            where.and(ConditionHelper.egalVarchar("T1.CODE", code));
        }
        RequeteSQL requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ctx, this, codeRubrique);
        final ClauseOrderBy orderBy = new ClauseOrderBy();
        orderBy.orderBy("T1.TITRE", SensDeTri.ASC);
        requeteSelect.orderBy(orderBy);
        final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ctx, nombre);
        requeteSelect.limit(limite);
        return select(requeteSelect.formaterRequete());
    }

    public List<Articleblog> getDerniersArticles(final OMContext ctx, final int offset, final int nb) throws Exception {
        final List<Articleblog> articles = new ArrayList<>();
        // peut-être à déplacer aussi
        try (ContexteDao ctxDao = new ContexteDao();
            PreparedStatement stmt = ctxDao.getConnection().prepareStatement("select ID_ARTICLEBLOG from ARTICLEBLOG where ETAT_OBJET = ? and CODE_RUBRIQUE = ? order by DATE_MODIFICATION desc limit ?, ?")) {
            stmt.setObject(1, "0003", Types.VARCHAR);
            stmt.setObject(2, getCode(), Types.VARCHAR);
            stmt.setObject(3, offset, Types.BIGINT);
            stmt.setObject(4, nb, Types.BIGINT);
            try (ResultSet rs = stmt.executeQuery()) {
                while (rs.next()) {
                    Articleblog article = new Articleblog();
                    article.init();
                    article.setIdArticleblog(rs.getLong("ID_ARTICLEBLOG"));
                    article.retrieve();
                    articles.add(article);
                }
            }
        }
        return articles;
    }

    public List<Articleblog> getKeywordArticles(final OMContext ctx, final String keyword) throws Exception {
        final List<Articleblog> articles = new ArrayList<>();
        try (ContexteDao ctxDao = new ContexteDao();
            PreparedStatement stmt = ctxDao.getConnection().prepareStatement("select ID_ARTICLEBLOG from ARTICLEBLOG inner join METATAG on ID_ARTICLEBLOG = META_ID_FICHE where META_CODE_OBJET = '9130' and ETAT_OBJET = ? and CODE_RUBRIQUE = ? and KEYWORDS like ? order by META_DATE_MISE_EN_LIGNE desc")) {
            stmt.setObject(1, "0003", Types.VARCHAR);
            stmt.setObject(2, getCode(), Types.VARCHAR);
            stmt.setObject(3, "%" + keyword + "%", Types.VARCHAR);
            try (ResultSet rs = stmt.executeQuery()) {
                while (rs.next()) {
                    Articleblog article = new Articleblog();
                    article.init();
                    article.setIdArticleblog(rs.getLong("ID_ARTICLEBLOG"));
                    article.retrieve();
                    articles.add(article);
                }
            }
        }
        return articles;
    }

    public List<Articleblog> getMonthArticles(final OMContext ctx, final java.util.Date date) throws Exception {
        final List<Articleblog> articles = new ArrayList<>();
        try (ContexteDao ctxDao = new ContexteDao();
            PreparedStatement stmt = ctxDao.getConnection().prepareStatement("select ID_ARTICLEBLOG from ARTICLEBLOG inner join METATAG on ID_ARTICLEBLOG = META_ID_FICHE where META_CODE_OBJET = '9130' and ETAT_OBJET = ? and CODE_RUBRIQUE = ? and META_DATE_MISE_EN_LIGNE like concat(substring(?, 1, 7), '%') order by META_DATE_MISE_EN_LIGNE desc")) {
            stmt.setObject(1, EtatFiche.EN_LIGNE.getEtat(), Types.VARCHAR);
            stmt.setObject(2, getCode(), Types.VARCHAR);
            stmt.setObject(3, SqlDateConverter.toDate(date), Types.DATE);
            try (ResultSet rs = stmt.executeQuery()) {
                while (rs.next()) {
                    Articleblog article = new Articleblog();
                    article.init();
                    article.setIdArticleblog(rs.getLong("ID_ARTICLEBLOG"));
                    article.retrieve();
                    articles.add(article);
                }
            }
        }
        return articles;
    }

    public List<Articleblog> getYearArticles(final OMContext ctx, final java.util.Date date) throws Exception {
        final List<Articleblog> articles = new ArrayList<>();
        try (ContexteDao ctxDao = new ContexteDao();
            PreparedStatement stmt = ctxDao.getConnection().prepareStatement("select ID_ARTICLEBLOG from ARTICLEBLOG inner join METATAG on ID_ARTICLEBLOG = META_ID_FICHE where META_CODE_OBJET = '9130' and ETAT_OBJET = ? and CODE_RUBRIQUE = ? and META_DATE_MISE_EN_LIGNE like concat(substring(?, 1, 4), '%') order by META_DATE_MISE_EN_LIGNE desc")) {
            stmt.setObject(1, EtatFiche.EN_LIGNE.getEtat(), Types.VARCHAR);
            stmt.setObject(2, getCode(), Types.VARCHAR);
            stmt.setObject(3, SqlDateConverter.toDate(date), Types.DATE);
            try (ResultSet rs = stmt.executeQuery()) {
                while (rs.next()) {
                    Articleblog article = new Articleblog();
                    article.init();
                    article.setIdArticleblog(rs.getLong("ID_ARTICLEBLOG"));
                    article.retrieve();
                    articles.add(article);
                }
            }
        }
        return articles;
    }

    public List<String> getKeywords() throws SQLException {
        final List<String> keywords = new ArrayList<>();
        try (ContexteDao ctxDao = new ContexteDao();
             PreparedStatement stmt = ctxDao.getConnection().prepareStatement("select KEYWORDS from ARTICLEBLOG where ETAT_OBJET = ? and CODE_RUBRIQUE = ?")) {
            stmt.setObject(1, EtatFiche.EN_LIGNE.getEtat(), Types.VARCHAR);
            stmt.setObject(2, getCode(), Types.VARCHAR);
            try (ResultSet rs = stmt.executeQuery()) {
                while (rs.next()) {
                    final String words = rs.getString("KEYWORDS");
                    try (final Scanner scanner = new Scanner(words)) {
                        scanner.useDelimiter("\\s*,\\s");
                        while (scanner.hasNext()) {
                            final String word = scanner.next();
                            if (word.trim().length() > 0 && !keywords.contains(word)) {
                                keywords.add(word);
                            }
                        }
                    }
                }
            }
        }
        return keywords;
    }

    public Map<String, Integer> getKeywordsCount() throws SQLException {
        final Map<String, Integer> keywords = new LinkedHashMap<>();
        try (ContexteDao ctxDao = new ContexteDao();
            PreparedStatement stmt = ctxDao.getConnection().prepareStatement("select KEYWORDS from ARTICLEBLOG where ETAT_OBJET = ? and CODE_RUBRIQUE = ?")) {
            stmt.setObject(1, EtatFiche.EN_LIGNE.getEtat(), Types.VARCHAR);
            stmt.setObject(2, getCode(), Types.VARCHAR);
            try (ResultSet rs = stmt.executeQuery()) {
                while (rs.next()) {
                    final String words = rs.getString("KEYWORDS");
                    try (final Scanner scanner = new Scanner(words)) {
                        scanner.useDelimiter("\\s*,\\s");
                        while (scanner.hasNext()) {
                            final String word = scanner.next();
                            if (word.trim().length() > 0) {
                                if (!keywords.containsKey(word)) {
                                    keywords.put(word, 1);
                                } else {
                                    keywords.put(word, keywords.get(word) + 1);
                                }
                            }
                        }
                    }
                }
            }
        }
        return keywords;
    }

    public List<java.util.Date> getMonths() throws SQLException {
        final List<java.util.Date> months = new ArrayList<>();
        final List<Long> mark = new ArrayList<>();
        try (ContexteDao ctxDao = new ContexteDao();
             PreparedStatement stmt = ctxDao.getConnection().prepareStatement("select DATE(META_DATE_MISE_EN_LIGNE) as DATE_PUBLICATION from ARTICLEBLOG inner join METATAG on ID_ARTICLEBLOG = META_ID_FICHE where META_CODE_OBJET = '9130' and ETAT_OBJET = ? and CODE_RUBRIQUE = ? order by META_DATE_MISE_EN_LIGNE desc")) {
            stmt.setObject(1, EtatFiche.EN_LIGNE.getEtat(), Types.VARCHAR);
            stmt.setObject(2, getCode(), Types.VARCHAR);
            try (ResultSet rs = stmt.executeQuery()) {
                while (rs.next()) {
                    final Calendar date = Calendar.getInstance();
                    date.setTime(rs.getDate("DATE_PUBLICATION"));
                    date.set(Calendar.DATE, 1);
                    if (!mark.contains(date.getTimeInMillis())) {
                        months.add(date.getTime());
                        mark.add(date.getTimeInMillis());
                    }
                }
            }
        }
        return months;
    }

    public List<java.util.Date> getYears() throws SQLException {
        final List<java.util.Date> months = new ArrayList<>();
        final List<Long> mark = new ArrayList<>();
        try (ContexteDao ctxDao = new ContexteDao();
            PreparedStatement stmt = ctxDao.getConnection().prepareStatement("select DATE(META_DATE_MISE_EN_LIGNE) as DATE_PUBLICATION from ARTICLEBLOG inner join METATAG on ID_ARTICLEBLOG = META_ID_FICHE where META_CODE_OBJET = '9130' and ETAT_OBJET = ? and CODE_RUBRIQUE = ? order by META_DATE_MISE_EN_LIGNE desc")) {
            stmt.setObject(1, EtatFiche.EN_LIGNE.getEtat(), Types.VARCHAR);
            stmt.setObject(2, getCode(), Types.VARCHAR);
            try (ResultSet rs = stmt.executeQuery()) {
                while (rs.next()) {
                    final Calendar date = Calendar.getInstance();
                    date.setTime(rs.getDate("DATE_PUBLICATION"));
                    date.set(Calendar.MONTH, 1);
                    date.set(Calendar.DATE, 1);
                    if (!mark.contains(date.getTimeInMillis())) {
                        months.add(date.getTime());
                        mark.add(date.getTimeInMillis());
                    }
                }
            }
        }
        return months;
    }

    public List<Articleblog> getArticles(final OMContext ctx) throws Exception {
        final List<Articleblog> articles = new ArrayList<>();
        try (ContexteDao ctxDao = new ContexteDao();
             PreparedStatement stmt = ctxDao.getConnection().prepareStatement("select ID_ARTICLEBLOG from ARTICLEBLOG inner join METATAG on ID_ARTICLEBLOG = META_ID_FICHE where META_CODE_OBJET = '9130' and ETAT_OBJET = ? and CODE_RUBRIQUE = ? order by DATE_MODIFICATION desc")) {
            stmt.setObject(1, EtatFiche.EN_LIGNE.getEtat(), Types.VARCHAR);
            stmt.setObject(2, getCode(), Types.VARCHAR);
            try (ResultSet rs = stmt.executeQuery()) {
                while (rs.next()) {
                    Articleblog article = new Articleblog();
                    article.init();
                    article.setIdArticleblog(rs.getLong("ID_ARTICLEBLOG"));
                    article.retrieve();
                    articles.add(article);
                }
            }
        }
        return articles;
    }

    public Date getLastUpdate() throws SQLException {
        Date lastUpdate = null;
        try (ContexteDao ctxDao = new ContexteDao();
            PreparedStatement stmt = ctxDao.getConnection().prepareStatement("select DATE_MODIFICATION from ARTICLEBLOG inner join METATAG on ID_ARTICLEBLOG = META_ID_FICHE where META_CODE_OBJET = '9130' and ETAT_OBJET = ? and CODE_RUBRIQUE = ? order by DATE_MODIFICATION desc")) {
            stmt.setObject(1, EtatFiche.EN_LIGNE.getEtat(), Types.VARCHAR);
            stmt.setObject(2, getCode(), Types.VARCHAR);
            try (ResultSet rs = stmt.executeQuery()) {
                if (rs.next()) {
                    lastUpdate = rs.getDate("DATE_MODIFICATION");
                }
            }
        }
        return lastUpdate;
    }

    public Long getIdBlog(){
        return persistenceBean.getId();
    }

    public void setIdBlog(Long id){
        persistenceBean.setId(id);
    }

    public String getDescription() {
        return persistenceBean.getDescription();
    }

    public void setDescription(String description) {
        this.persistenceBean.setDescription(description);
    }

    public String getThematique() {
        return persistenceBean.getThematique();
    }

    public void setThematique(String thematique) {
        this.persistenceBean.setThematique(thematique);
    }

    public String getArchivage() {
        return persistenceBean.getArchivage();
    }

    public void setArchivage(String archivage) {
        this.persistenceBean.setArchivage(archivage);
    }
}
