/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: blog
 * Version: 1.09.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.blog.util;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import com.jsbsoft.jtf.core.LangueUtil;
import com.univ.utils.ContexteDao;

/**
 * Compléments DAO 
 * @author emmanuel.clisson
 *
 */
public abstract class FicheUtil {

    public static boolean exists(String nomobjet, String code) throws Exception {
        return exists(nomobjet, code, Integer.toString(LangueUtil.getIndiceLocale(LangueUtil.getDefaultLocale())));
    }

    public static boolean exists(String nomobjet, String code, String langue) throws Exception {
        return exists(nomobjet, code, langue, "0003");
    }

    public static boolean exists(String nomobjet, String code, String langue, String etat) throws Exception {
        boolean alreadyExists = false;
        ResultSet rs = null;
        String query = "select * from " + nomobjet.toUpperCase() + " where CODE = ? and LANGUE = ? and ETAT_OBJET = ?";
        try (ContexteDao ctxDao = new ContexteDao();
             PreparedStatement stmt = ctxDao.getConnection().prepareStatement(query);) {
            stmt.setObject(1, code, Types.VARCHAR);
            stmt.setObject(2, langue, Types.VARCHAR);
            stmt.setObject(3, etat, Types.VARCHAR);
            rs = stmt.executeQuery();
            if (rs.next()) {
                alreadyExists = true;
            }
        } finally {
            if (rs != null) {
                try {
                    rs.close();
                } catch (SQLException ignored) {
                }
            }
        }
        return alreadyExists;
    }
}
